@@ -296,12 +296,18 @@ var UI = {
     }
     if (tar_only==0 || tar_only==-1){
       let pinfo = await UI.calculator.getPlatformInfo();
+      let sector=(Math.floor(UI.calculator.device.endslot/65536)+1);
+      let appstart=0x90000000+sector*65536;
       let n120=UI.calculator.device.startAddress >= 0x24000000;
-      console.log('install_slotb versio_longue tar_only',version_longue,tar_only);
+      let appfile='sector'+sector.toString(16)+'v'+UI.calculator.device.epsilon+(n120?'.120':'.110');
+      console.log('install_slotb version_longue tar_only',version_longue,tar_only);
+      console.log('file transfered',appfile,'1st external app address','0x'+appstart.toString(16));
+      UI.print('file transfered '+appfile+', 1st external app address 0x'+appstart.toString(16));
       if (tar_only==-1){
-	UI.calculator.device.startAddress = n120?0x90190000:0x90180000;
-	UI.calculator.device.logProgress(0,'KhiCAS launcher at '+(n120?'0x90190000':'0x90180000'));
-	let data=await UI.loadfile(n120?"sector19":"sector18");
+	UI.calculator.device.startAddress = appstart; // n120?0x90190000:0x90180000;
+	UI.calculator.device.logProgress(0,'KhiCAS launcher at '+appstart);
+	let data=await UI.loadfile(appfile);
+	// let data=await UI.loadfile(n120?"sector19":"sector18");
 	let res=await UI.calculator.device.do_download(UI.calculator.transferSize, data, false);
 	if (res==0){
 	  alert(UI.langue==-1?'Erreur d\'ecriture flash':'Error writing flash');
@@ -459,6 +465,86 @@ var UI = {
       return response.arrayBuffer();
     })
   },
+  numworks_activateapps_:async function(){
+    if (UI.calculator==0 || !UI.calculator_connected){
+      alert(UI.langue==-1?UI.chkmsgfr:chkmsgen);
+      //console.log('numworks_rescue_',UI.calculator);
+      if (UI.calculator) UI.calculator.stopAutoConnect();
+      UI.calculator=0;
+      return -1;
+    }
+    // UI.print(UI.langue==-1?'Rendre les applications externes visibles fonctionne avec Epsilon 19.5':'Works with Epsilon 19.5');
+const address = 0x90200000;
+// Commande DFUse (via DFU_DNLOAD)
+const buf = new ArrayBuffer(5);
+const view = new DataView(buf);
+view.setUint8(0, 0x21);  // SET_ADDRESS_POINTER (commande DFUse)
+view.setUint32(1, 0x90200000, true);
+
+await UI.calculator.device.device_.controlTransferOut({
+    requestType: 'class',
+    recipient: 'interface',
+    request: 0x01,  // DFU_DNLOAD (commande DFU standard)
+    value: 0,
+    index: 0
+}, buf);
+
+// 2. Attendre et vérifier le statut
+await new Promise(r => setTimeout(r, 50));
+
+let st=await UI.calculator.device.device_.controlTransferIn({
+    requestType: 'class',
+    recipient: 'interface',
+    request: 0x03,      // DFU_GETSTATUS
+    value: 0,
+    index: 0
+}, 6);  // 6 bytes de statut DFU
+      console.log(st);
+      
+st=await UI.calculator.device.device_.controlTransferIn({
+    requestType: 'class',
+    recipient: 'interface',
+    request: 0x03,      // DFU_GETSTATUS
+    value: 0,
+    index: 0
+}, 6);  // 6 bytes de statut DFU
+      console.log(st);      
+
+      
+// 3. Paquet vide avec wValue=0 pour terminer la commande SET_ADDRESS
+await UI.calculator.device.device_.controlTransferOut({
+    requestType: 'class',
+    recipient: 'interface',
+    request: 0x01,      // DFU_DNLOAD
+    value: 2,           // wValue = 2 for leave
+    index: 0
+}, new ArrayBuffer(0));
+      
+st=await UI.calculator.device.device_.controlTransferIn({
+    requestType: 'class',
+    recipient: 'interface',
+    request: 0x03,      // DFU_GETSTATUS
+    value: 0,
+    index: 0
+}, 6);  // 6 bytes de statut DFU
+      console.log(st);
+      return;
+// 4. :leave - DFU_DETACH
+await UI.calculator.device.device_.controlTransferOut({
+    requestType: 'class',
+    recipient: 'interface',
+    request: 0x00,      // DFU_DETACH
+    value: 0,
+    index: 0
+});
+  },
+  numworks_activateapps:async function(){
+    if (UI.calculator==0 || !UI.calculator_connected){
+      let res=await UI.nws_connect();
+      if (res==-1) return;
+    }
+    window.setTimeout(UI.numworks_activateapps_,100);
+  },
   numworks_rescue:async function(){
     alert(UI.langue==-1?'Connectez la calculatrice, appuyez sur la touche 6 de la calculatrice, enfoncez un stylo dans le bouton RESET au dos en laissant la touche 6 appuyee, relachez la touche 6, l\'ecran doit etre eteint et la diode rouge allumee':'Connect the calculator, press the 6 key on the calculator, press the RESET button on the back keeping the 6 key pressed, release the 6 key, the screen should be down and the led should be red');
     UI.calc=2;
@@ -1163,7 +1249,6 @@ var UI = {
         alert(UI.langue==-1?'Kernel/Userland slot A non certifiés.':'Kernel/Userland slot A not certified');
         return 2;
       }
-      UI.print('External flash Epsilon slot A OK');
       // ok, check full KhiCAS install at 0x90260000
       external=await UI.calculator.get_external_flash(0x26);
       res=await UI.sig_check(sigfile,external,'khi110a');
@@ -1171,14 +1256,12 @@ var UI = {
         alert(UI.langue==-1?'KhiCAS app A non certifiée.':'KhiCAS app A not certified');
         return 2;
       }
-      UI.print('External flash KhiCAS part A OK');
       external=await UI.calculator.get_external_flash(3);
       res=await UI.sig_check(sigfile,external,'khi110ab.tar');
       if (!res){
         alert(UI.langue==-1?'KhiCAS app B non certifiée.':'KhiCAS app B not certified');
         return 2;
       }
-      UI.print('External flash KhiCAS part B OK');
       if (rwcheck){
 	// do random r/w starting at 0x1c0000
 	UI.print('R/W check 0x901a00000');
@@ -1187,7 +1270,6 @@ var UI = {
 	  alert(UI.langue==-1?'Echec du test lecture/ecriture 0x901a0000':'Read/Write test failure 0x901c0000');
 	  return 4;
 	}
-	UI.print('R/W check 0x901a00000 OK');
       }
       alert(UI.langue==-1?'Firmware certifie (externe et KhiCAS)':'Firmware certified (external and KhiCAS).');      
       return 0;
